@@ -1,9 +1,11 @@
+import os
+import subprocess
 import itertools
 from AppiumLibrary import AppiumLibrary
 from appium import webdriver
+from psutil import Process, NoSuchProcess
 from robot.api.deco import keyword
 from robot.libraries.BuiltIn import BuiltIn
-import subprocess
 
 from selenium.common.exceptions import NoSuchElementException
 from selenium.webdriver.common.action_chains import ActionChains
@@ -13,6 +15,36 @@ zoomba = BuiltIn()
 SCREENSHOT_COUNTER = itertools.count()
 
 
+class WinAppDriver:
+    def __init__(self, driver_path=""):
+        self.process = None
+        self.driver_path = driver_path
+
+    def set_up_driver(self, path=None):
+        if path is None:
+            path = self.driver_path
+        try:
+            stdout = open(os.devnull, 'w')
+            self.process = subprocess.Popen([path], stdout=stdout)
+            stdout.close()
+        except Exception:
+            self.process = None
+            stdout.close()
+
+    def tear_down_driver(self):
+        try:
+            process = Process(self.process.pid)
+            for pro in process.children(recursive=True):
+                pro.kill()
+                pro.wait()
+            self.process.kill()
+            self.process.wait()
+            self.process = None
+        except (NoSuchProcess, AttributeError):
+            subprocess.call("C:/Windows/system32/taskkill.exe /f /im WinAppDriver.exe", shell=False)
+            self.process = None
+
+
 class DesktopLibrary(AppiumLibrary):
     """Zoomba Desktop Library
 
@@ -46,20 +78,28 @@ class DesktopLibrary(AppiumLibrary):
     ``Wait Until Page Contains`` or ``Wait Until Page Contains Element``.
     """
 
-    def __init__(self, timeout=5, run_on_failure='Save Appium Screenshot'):
+    def __init__(self, timeout=5, run_on_failure='Save Appium Screenshot',
+                 driver_path="C:\\Program Files (x86)\\Windows Application Driver\\WinAppDriver.exe"):
         """DesktopLibrary can be imported with optional arguments.
+
         ``timeout`` is the default timeout used to wait for all waiting actions.
         It can be later set with `Set Appium Timeout`.
+
         ``run_on_failure`` specifies the name of a keyword (from any available
         libraries) to execute when a DesktopLibrary keyword fails.
         By default `Save Appium Screenshot` will be used to take a screenshot of the current page.
-        Using the value `No Operation` will disable this feature altogether. See
+        Using the value ``No Operation`` will disable this feature altogether. See
         `Register Keyword To Run On Failure` keyword for more information about this
         functionality.
+
+        ``driver_path`` is the path to the WinAppDriver.exe file.
+
         Examples:
         | Library | DesktopLibrary | 10 | # Sets default timeout to 10 seconds                                                                             |
         | Library | DesktopLibrary | timeout=10 | run_on_failure=No Operation | # Sets default timeout to 10 seconds and does nothing on failure           |
+        | Library | DesktopLibrary | timeout=10 | driver_path="C:/WinAppDriver.exe" | # Sets a new path for the WinAppDriver                               |
         """
+        self.winappdriver = WinAppDriver(driver_path)
         super().__init__(timeout, run_on_failure)
 
     def get_keyword_names(self):
@@ -68,36 +108,50 @@ class DesktopLibrary(AppiumLibrary):
         libraries from being referenced and used.
         """
         return [
-            'maximize_window', 'open_application', 'wait_for_and_clear_text', 'wait_for_and_click_element',
-            'wait_for_and_click_text', 'wait_for_and_input_password', 'wait_for_and_input_text',
-            'wait_for_and_long_press', 'wait_until_element_contains', 'wait_until_element_does_not_contain',
-            'wait_until_element_is_enabled', 'wait_until_element_is_disabled', 'switch_application_by_name',
-            'mouse_over_element', 'wait_for_and_mouse_over_element', 'mouse_over_and_click_element',
-            'wait_for_and_mouse_over_and_click_element', 'mouse_over_text', 'wait_for_and_mouse_over_text',
-            'mouse_over_and_click_text', 'wait_for_and_mouse_over_and_click_text', 'click_a_point',
-            'context_click_a_point', 'mouse_over_and_context_click_element', 'mouse_over_and_context_click_text',
-            'mouse_over_by_offset', 'drag_and_drop', 'drag_and_drop_by_offset', 'send_keys', 'send_keys_to_element',
-            'capture_page_screenshot', 'save_appium_screenshot', 'select_element_from_combobox', 'get_source',
-            'click_text',
+            'maximize_window', 'open_application', 'wait_for_and_clear_text',
+            'wait_for_and_click_element', 'wait_for_and_input_password', 'wait_for_and_input_text',
+            'wait_for_and_long_press', 'wait_until_element_contains',
+            'wait_until_element_does_not_contain', 'wait_until_element_is_enabled',
+            'wait_until_element_is_disabled', 'switch_application_by_name', 'mouse_over_element',
+            'wait_for_and_mouse_over_element', 'mouse_over_and_click_element',
+            'wait_for_and_mouse_over_and_click_element', 'click_a_point', 'context_click_a_point',
+            'mouse_over_and_context_click_element', 'mouse_over_by_offset', 'drag_and_drop',
+            'drag_and_drop_by_offset', 'send_keys', 'send_keys_to_element',
+            'capture_page_screenshot', 'save_appium_screenshot', 'select_element_from_combobox',
+            'driver_setup', 'driver_teardown',
             # External Libraries
-            'clear_text', 'click_button', 'click_element', 'close_all_applications', 'close_application',
-            'element_attribute_should_match', 'element_should_be_disabled', "element_should_be_enabled",
-            'element_should_be_visible', 'element_should_contain_text', 'element_should_not_contain_text',
-            'element_text_should_be', 'get_appium_sessionId', 'get_appium_timeout', 'get_capability',
-            'get_element_attribute', 'get_element_location', 'get_element_size', 'get_webelement',
-            'get_webelements', 'get_window_height', 'get_window_width', 'input_password', 'input_text',
-            'launch_application', 'log_source', 'long_press', 'page_should_contain_element', 'page_should_contain_text',
+            'clear_text', 'click_button', 'click_element', 'close_all_applications',
+            'close_application', 'element_attribute_should_match', 'element_should_be_disabled',
+            "element_should_be_enabled", 'element_should_be_visible', 'element_should_contain_text',
+            'element_should_not_contain_text', 'element_text_should_be', 'get_appium_sessionId',
+            'get_appium_timeout', 'get_capability', 'get_element_attribute', 'get_element_location',
+            'get_element_size', 'get_webelement', 'get_webelements', 'get_window_height',
+            'get_window_width', 'input_password', 'input_text', 'launch_application', 'log_source',
+            'long_press', 'page_should_contain_element', 'page_should_contain_text',
             'page_should_not_contain_element', 'page_should_not_contain_text', 'quit_application',
-            'register_keyword_to_run_on_failure', 'set_appium_timeout', 'switch_application', 'text_should_be_visible',
-            'wait_until_element_is_visible', 'wait_until_page_contains', 'wait_until_page_contains_element',
-            'wait_until_page_does_not_contain', 'wait_until_page_does_not_contain_element', 'get_matching_xpath_count',
+            'register_keyword_to_run_on_failure', 'set_appium_timeout', 'switch_application',
+            'text_should_be_visible', 'wait_until_element_is_visible', 'wait_until_page_contains',
+            'wait_until_page_contains_element', 'wait_until_page_does_not_contain',
+            'wait_until_page_does_not_contain_element', 'get_matching_xpath_count',
             'xpath_should_match_x_times'
         ]
 
+    @keyword("Driver Setup")
+    def driver_setup(self, path=None):
+        """Starts the WinAppDriver.
+
+        ``path`` can be provided if your winappdriver intallation is not in the default path of
+        ``C:/Program Files (x86)/Windows Application Driver/WinAppDriver.exe``."""
+        self.winappdriver.set_up_driver(path)
+
+    @keyword("Driver Teardown")
+    def driver_teardown(self):
+        """Stops the WinAppDriver."""
+        self.winappdriver.tear_down_driver()
+
     @keyword("Maximize Window")
     def maximize_window(self):
-        """Maximizes the current application window.
-        """
+        """Maximizes the current application window."""
         driver = self._current_application()
         driver.maximize_window()
         return True
@@ -247,19 +301,6 @@ class DesktopLibrary(AppiumLibrary):
         self._wait_until_page_contains_element(locator, timeout, error)
         self.click_element(locator)
 
-    @keyword("Click Text")
-    def click_text(self, text, exact_match=False):
-        """*DEPRECATED in DesktopLibrary 2.4.0* Use `Click Element` with the ``name`` prefix instead.
-        """
-        self._element_find_by_text(text, exact_match).click()
-
-    @keyword("Wait For And Click Text")
-    def wait_for_and_click_text(self, text, exact_match=False, timeout=None, error=None):
-        """*DEPRECATED in DesktopLibrary 2.4.0* Use `Wait For And Click Element` with the ``name`` prefix instead.
-        """
-        self._wait_until_page_contains(text, timeout, error)
-        self.click_text(text, exact_match)
-
     @keyword("Wait For And Input Password")
     def wait_for_and_input_password(self, locator, text, timeout=None, error=None):
         """Wait for and type the given password into the text field identified by ``locator``.
@@ -417,47 +458,6 @@ class DesktopLibrary(AppiumLibrary):
         self._wait_until_page_contains_element(locator, timeout, error)
         self.mouse_over_and_click_element(locator, double_click, x_offset, y_offset)
 
-    @keyword("Mouse Over Text")
-    def mouse_over_text(self, text, exact_match=False, x_offset=0, y_offset=0):
-        """*DEPRECATED in DesktopLibrary 2.4.0* Use `Mouse Over Element` with the ``name`` prefix instead.
-        """
-        driver = self._current_application()
-        element = self._element_find_by_text(text, exact_match)
-        actions = ActionChains(driver)
-        self._move_to_element(actions, element, x_offset, y_offset)
-        actions.perform()
-
-    @keyword("Wait For And Mouse Over Text")
-    def wait_for_and_mouse_over_text(self, text, exact_match=False, timeout=None, error=None, x_offset=0, y_offset=0):
-        """*DEPRECATED in DesktopLibrary 2.4.0* Use `Wait For And Mouse Over Element` with the ``name`` prefix instead.
-        """
-        self._wait_until_page_contains(text, timeout, error)
-        self.mouse_over_text(text, exact_match, x_offset, y_offset)
-
-    @keyword("Mouse Over And Click Text")
-    def mouse_over_and_click_text(self, text, exact_match=False, double_click=False, x_offset=0, y_offset=0):
-        """*DEPRECATED in DesktopLibrary 2.4.0* Use `Mouse Over And Click Element` with the ``name`` prefix instead.
-        """
-        self.mouse_over_text(text, exact_match=exact_match, x_offset=x_offset, y_offset=y_offset)
-        self.click_a_point(double_click=double_click)
-
-    @keyword("Mouse Over And Context Click Text")
-    def mouse_over_and_context_click_text(self, text, exact_match=False, x_offset=0, y_offset=0):
-        """*DEPRECATED in DesktopLibrary 2.4.0* Use `Mouse Over And Context Click Element` with the ``name``
-        prefix instead.
-        """
-        self.mouse_over_text(text, exact_match=exact_match, x_offset=x_offset, y_offset=y_offset)
-        self.context_click_a_point()
-
-    @keyword("Wait For And Mouse Over And Click Text")
-    def wait_for_and_mouse_over_and_click_text(self, text, exact_match=False, timeout=None, error=None,
-                                               double_click=False, x_offset=0, y_offset=0):
-        """*DEPRECATED in DesktopLibrary 2.4.0* Use `Wait For And Mouse Over And Click Element` with the ``name``
-         prefix instead.
-        """
-        self._wait_until_page_contains(text, timeout, error)
-        self.mouse_over_and_click_text(text, exact_match, double_click, x_offset, y_offset)
-
     @keyword("Mouse Over By Offset")
     def mouse_over_by_offset(self, x_offset=0, y_offset=0):
         """Moves the mouse from its current location by the given ``x_offset`` and ``y_offset``.
@@ -629,13 +629,6 @@ class DesktopLibrary(AppiumLibrary):
             self.switch_application(original_index)
 
     # Private
-    def _element_find_by_text(self, text, exact_match=False):
-        if exact_match:
-            _xpath = u'//*[@{}="{}"]'.format('Name', text)
-        else:
-            _xpath = u'//*[contains(@{},"{}")]'.format('Name', text)
-        return self._element_find(_xpath, True, True)
-
     def _move_to_element(self, actions, element, x_offset=0, y_offset=0):
         if x_offset != 0 or y_offset != 0:
             self._info('Moving to element "' + str(element) + '" with offset (%s,%s).' % (x_offset, y_offset))
@@ -693,12 +686,6 @@ class DesktopLibrary(AppiumLibrary):
                 prefix = locator_parts[0].strip().lower()
                 criteria = locator_parts[2].strip()
         return prefix, criteria
-    
-    def _wait_until_page_contains(self, text, timeout=None, error=None):
-        """Internal version to avoid duplicate screenshots"""
-        if not error:
-            error = "Text '%s' did not appear in <TIMEOUT>" % text
-        self._wait_until(timeout, error, self._is_text_present, text)
 
     def _wait_until_page_contains_element(self, locator, timeout=None, error=None):
         """Internal version to avoid duplicate screenshots"""
